"""
A set of variables used for testing the relayr API.
Attention: You must provide your own eaningful values 
#before you can run large parts of the test suite!
"""

testset0 = {
    'userEmail': 'god@in.heaven',
    'relayrAPI': 'https://api.relayr.io'
}
