# A set of variables used for testing the relayr API.
# Attention: You must provide your own eaningful values 
# before you can run large parts of the test suite!

# Get 'token' on relayr.io!

# Remember that tokens expire after some time, check the documentation!

testset1 = {
    'userID': 'd320f267-4bba-433d-a59a-2e9fc881df8f', 
    'userName': 'dinugherman', 
    'userEmail': 'gherman@darwin.in-berlin.de',
    'relayrAPI': 'https://api.relayr.io',
    'redirectURI': 'https://relayr.io',
    'response_type': 'code', # or 'token'
    'clientId': 'CzmPlGbCtUI8tAwvlSJkW8IcRjazCp6u',
    'clientSecret': '05UlzL2r3zFf7aeAPmZMOF2wnBHdd5Ue',
    'scope': 'access-own-user-info',
    'appID': 'c6941a2b-3b2f-404f-962d-cd88328eff26',
    'appName': 'DGApp0',
    'publisherID': 'f4002d7d-f6e9-46b8-b79a-dbb6c435f282',
    'deviceID': 'a3386628-61a8-4a80-9a42-1bd3c64c46a5',
    'description': "Dinu's first Wunderbar app",
    'useragent': 'io.relayr.postman/0.1.0',
    'token': 'KkD_V4TnjmN7Mw_zceRhZuNf1GCG_N9X',
    'transmitterID': '4f3b23be-53b7-4b1d-8da7-379a0b81eeba',
    'transmitterName': 'DCG08 My Wunderbar',
}

testset3 = {
    'PUBLISH_KEY': 'pub-c-26992c4a-5a1c-4ae1-ba04-bb01f87ec12b',
    'SUBSCRIBE_KEY': 'sub-c-b702a884-43fe-11e4-a249-02ee2ddab7fe',
    'channel': 'test-channel',
    'message': 'hi there 11'
}

testset4 = {
    'bluetoothAddr': 'f1:42:e6:63:20:a2',
}
